Package: ggrepel
Version: 0.9.8
Authors@R: c(
    person("Kamil", "Slowikowski", email = "kslowikowski@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-2843-6370")),
    person("Teun", "van den Brand", role = "ctb", comment = c(ORCID = "0000-0002-9335-7468")),
    person("Alicia", "Schep", role = "ctb", comment = c(ORCID = "0000-0002-3915-0618")),
    person("Sean", "Hughes", role = "ctb", comment = c(ORCID = "0000-0002-9409-9405")),
    person("Trung Kien", "Dang", role = "ctb", comment = c(ORCID = "0000-0001-7562-6495")),
    person("Saulius", "Lukauskas", role = "ctb"),
    person("Jean-Olivier", "Irisson", role = "ctb", comment = c(ORCID = "0000-0003-4920-3880")),
    person("Zhian N", "Kamvar", role = "ctb", comment = c(ORCID = "0000-0003-1458-7108")),
    person("Thompson", "Ryan", role = "ctb", comment = c(ORCID = "0000-0002-0450-8181")),
    person("Dervieux", "Christophe", role = "ctb", comment = c(ORCID = "0000-0003-4474-2498")),
    person("Yutani", "Hiroaki", role = "ctb"),
    person("Pierre", "Gramme", role = "ctb"),
    person("Amir Masoud", "Abdol", role = "ctb"),
    person("Malcolm", "Barrett", role = "ctb", comment = c(ORCID = "0000-0003-0299-5825")),
    person("Robrecht", "Cannoodt", role = "ctb", comment = c(ORCID = "0000-0003-3641-729X")),
    person("Michał", "Krassowski", role = "ctb", comment = c(ORCID = "0000-0002-9638-7785")),
    person("Michael", "Chirico", role = "ctb", comment = c(ORCID = "0000-0003-0787-087X")),
    person("Pedro", "Aphalo", role = "ctb", comment = c(ORCID = "0000-0003-3385-972X")),
    person("Francis", "Barton", role = "ctb")
    )
Title: Automatically Position Non-Overlapping Text Labels with
        'ggplot2'
Description: Provides text and label geoms for 'ggplot2' that help to avoid
    overlapping text labels. Labels repel away from each other and away from the
    data points.
Depends: R (>= 4.1.0), ggplot2 (>= 3.5.2)
Imports: grid, Rcpp, rlang (>= 1.1.6), S7, scales (>= 1.4.0), withr (>=
        3.0.2)
Suggests: knitr, rmarkdown, testthat, svglite, vdiffr, gridExtra, ggpp,
        patchwork, devtools, prettydoc, ggbeeswarm, dplyr, magrittr,
        readr, stringr, marquee, rsvg, sf
VignetteBuilder: knitr
License: GPL-3 | file LICENSE
URL: https://ggrepel.slowkow.com/, https://github.com/slowkow/ggrepel
BugReports: https://github.com/slowkow/ggrepel/issues
RoxygenNote: 7.3.3
LinkingTo: Rcpp
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2026-03-16 17:37:05 UTC; ks38
Author: Kamil Slowikowski [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2843-6370>),
  Teun van den Brand [ctb] (ORCID:
    <https://orcid.org/0000-0002-9335-7468>),
  Alicia Schep [ctb] (ORCID: <https://orcid.org/0000-0002-3915-0618>),
  Sean Hughes [ctb] (ORCID: <https://orcid.org/0000-0002-9409-9405>),
  Trung Kien Dang [ctb] (ORCID: <https://orcid.org/0000-0001-7562-6495>),
  Saulius Lukauskas [ctb],
  Jean-Olivier Irisson [ctb] (ORCID:
    <https://orcid.org/0000-0003-4920-3880>),
  Zhian N Kamvar [ctb] (ORCID: <https://orcid.org/0000-0003-1458-7108>),
  Thompson Ryan [ctb] (ORCID: <https://orcid.org/0000-0002-0450-8181>),
  Dervieux Christophe [ctb] (ORCID:
    <https://orcid.org/0000-0003-4474-2498>),
  Yutani Hiroaki [ctb],
  Pierre Gramme [ctb],
  Amir Masoud Abdol [ctb],
  Malcolm Barrett [ctb] (ORCID: <https://orcid.org/0000-0003-0299-5825>),
  Robrecht Cannoodt [ctb] (ORCID:
    <https://orcid.org/0000-0003-3641-729X>),
  Michał Krassowski [ctb] (ORCID:
    <https://orcid.org/0000-0002-9638-7785>),
  Michael Chirico [ctb] (ORCID: <https://orcid.org/0000-0003-0787-087X>),
  Pedro Aphalo [ctb] (ORCID: <https://orcid.org/0000-0003-3385-972X>),
  Francis Barton [ctb]
Maintainer: Kamil Slowikowski <kslowikowski@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-17 08:30:38 UTC
