# Generated by roxygen2: do not edit by hand

S3method(element_grob,element_text_repel)
S3method(heightDetails,element_textrepeltree)
S3method(makeContent,labelrepeltree)
S3method(makeContent,marqueerepeltree)
S3method(makeContent,textrepeltree)
S3method(widthDetails,element_textrepeltree)
export(GeomLabelRepel)
export(GeomMarqueeRepel)
export(GeomTextRepel)
export(PositionNudgeRepel)
export(element_text_repel)
export(geom_label_repel)
export(geom_marquee_repel)
export(geom_text_repel)
export(position_nudge_repel)
import(Rcpp)
import(ggplot2)
importFrom(S7,S7_class)
importFrom(S7,class_any)
importFrom(S7,new_class)
importFrom(S7,new_property)
importFrom(S7,prop)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,curveGrob)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grobHeight)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,grobX)
importFrom(grid,grobY)
importFrom(grid,heightDetails)
importFrom(grid,is.grob)
importFrom(grid,is.unit)
importFrom(grid,makeContent)
importFrom(grid,resolveHJust)
importFrom(grid,resolveVJust)
importFrom(grid,roundrectGrob)
importFrom(grid,segmentsGrob)
importFrom(grid,setChildren)
importFrom(grid,stringHeight)
importFrom(grid,stringWidth)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,widthDetails)
importFrom(rlang,warn)
useDynLib(ggrepel)
